@@ -4,7 +4,7 @@ use strict;
 use warnings;
 
 BEGIN {
-    $Lego::From::PNG::VERSION = '0.02';
+    $Lego::From::PNG::VERSION = '0.03';
 }
 
 use Image::PNG::Libpng qw(:all);
@@ -16,6 +16,9 @@ use Lego::From::PNG::Brick;
 
 use Data::Debug;
 
+use Memoize;
+memoize('_find_lego_color', INSTALL => '_find_lego_color_fast');
+
 sub new {
     my $class = shift;
     my %args = ref $_[0] eq 'HASH' ? %{$_[0]} : @_;
@@ -37,11 +40,75 @@ sub new {
     # Black list default
     $hash->{'blacklist'} = ($args{'blacklist'} && ref($args{'blacklist'}) eq 'ARRAY' && scalar(@{$args{'blacklist'}}) > 0) ? $args{'blacklist'} : undef;
 
+    # Dimension measurement formats
+    $hash->{'metric'} = $args{'metric'} || 0;
+
+    $hash->{'imperial'} = $args{'imperial'} || 0;
+
+    $hash->{'metric'} = 1 if ! $hash->{'metric'} && ! $hash->{'imperial'};
+
     my $self = bless ($hash, ref ($class) || $class);
 
     return $self;
 }
 
+sub lego_dimensions {
+    my $self = shift;
+
+    return $self->{'lego_dimensions'} ||= do {
+        my $hash = {};
+
+        for my $type (qw/imperial metric/) {
+            my $lego_unit_length =
+                Lego::From::PNG::Const->LEGO_UNIT
+                * Lego::From::PNG::Const->LEGO_UNIT_LENGTH
+                * ($type eq 'imperial' ? Lego::From::PNG::Const->MILLIMETER_TO_INCH : 1);
+
+            my $lego_unit_depth =
+                Lego::From::PNG::Const->LEGO_UNIT
+                * Lego::From::PNG::Const->LEGO_UNIT_DEPTH
+                * ($type eq 'imperial' ? Lego::From::PNG::Const->MILLIMETER_TO_INCH : 1);
+
+            my $lego_unit_height =
+                Lego::From::PNG::Const->LEGO_UNIT
+                * Lego::From::PNG::Const->LEGO_UNIT_HEIGHT
+                * ($type eq 'imperial' ? Lego::From::PNG::Const->MILLIMETER_TO_INCH : 1);
+
+            my $lego_unit_stud_diameter =
+                Lego::From::PNG::Const->LEGO_UNIT
+                * Lego::From::PNG::Const->LEGO_UNIT_STUD_DIAMETER
+                * ($type eq 'imperial' ? Lego::From::PNG::Const->MILLIMETER_TO_INCH : 1);
+
+            my $lego_unit_stud_height =
+                Lego::From::PNG::Const->LEGO_UNIT
+                * Lego::From::PNG::Const->LEGO_UNIT_STUD_HEIGHT
+                * ($type eq 'imperial' ? Lego::From::PNG::Const->MILLIMETER_TO_INCH : 1);
+
+            my $lego_unit_stud_spacing =
+                Lego::From::PNG::Const->LEGO_UNIT
+                * Lego::From::PNG::Const->LEGO_UNIT_STUD_SPACING
+                * ($type eq 'imperial' ? Lego::From::PNG::Const->MILLIMETER_TO_INCH : 1);
+
+            my $lego_unit_edge_to_stud =
+                Lego::From::PNG::Const->LEGO_UNIT
+                * Lego::From::PNG::Const->LEGO_UNIT_EDGE_TO_STUD
+                * ($type eq 'imperial' ? Lego::From::PNG::Const->MILLIMETER_TO_INCH : 1);
+
+            $hash->{$type} = {
+                lego_unit_length        => $lego_unit_length,
+                lego_unit_depth         => $lego_unit_depth,
+                lego_unit_height        => $lego_unit_height,
+                lego_unit_stud_diameter => $lego_unit_stud_diameter,
+                lego_unit_stud_height   => $lego_unit_stud_height,
+                lego_unit_stud_spacing  => $lego_unit_stud_spacing,
+                lego_unit_edge_to_stud  => $lego_unit_edge_to_stud,
+            };
+        }
+
+        $hash;
+    };
+}
+
 sub lego_colors {
     my $self = shift;
 
@@ -111,10 +178,16 @@ sub png_info {
     return $self->{'png_info'} ||= $self->png->get_IHDR;
 }
 
-sub block_row_width {
+sub block_row_length {
     my $self = shift;
 
-    return $self->{'block_row_width'} ||= $self->png_info->{'width'} / $self->{'unit_size'};
+    return $self->{'block_row_length'} ||= $self->png_info->{'width'} / $self->{'unit_size'};
+}
+
+sub block_row_height {
+    my $self = shift;
+
+    return $self->{'block_row_height'} ||= $self->png_info->{'height'} / $self->{'unit_size'};
 }
 
 sub process {
@@ -150,6 +223,8 @@ sub process {
         }
 
         $tally->{'bricks'} = \%list;
+
+        $tally->{'info'} = $self->_plan_info();
     }
 
     if($args{'view'}) {
@@ -169,6 +244,76 @@ sub process {
     return $tally;
 }
 
+sub whitelist { shift->{'whitelist'} }
+
+sub has_whitelist {
+    my $self    = shift;
+    my $allowed = shift; # arrayref listing filters we can use
+
+    my $found = 0;
+    for my $filter(values $self->_list_filters($allowed)) {
+        $found += scalar( grep { /$filter/ } @{ $self->whitelist || [] } );
+    }
+
+    return $found;
+}
+
+sub is_whitelisted {
+    my $self    = shift;
+    my $val     = shift;
+    my $allowed = shift; # arrayref listing filters we can use
+
+    return 1 if ! $self->has_whitelist($allowed); # return true if there is no whitelist
+
+    for my $entry( @{ $self->whitelist || [] } ) {
+        for my $filter( values %{ $self->_list_filters($allowed) } ) {
+            next unless $entry =~ /$filter/; # if there is at least a letter at the beginning then this entry has a color we can check
+
+            my $capture = $entry;
+            $capture =~ s/$filter/$1/;
+
+            return 1 if $val eq $capture;
+        }
+    }
+
+    return 0; # value is not in whitelist
+}
+
+sub blacklist { shift->{'blacklist'} }
+
+sub has_blacklist {
+    my $self    = shift;
+    my $allowed = shift; # optional filter restriction
+
+    my $found = 0;
+
+    for my $filter(values $self->_list_filters($allowed)) {
+        $found += scalar( grep { /$filter/ } @{ $self->blacklist || [] } );
+    }
+
+    return $found;
+}
+
+sub is_blacklisted {
+    my $self    = shift;
+    my $val     = shift;
+    my $allowed = shift; # optional filter restriction
+
+    return 0 if ! $self->has_blacklist($allowed); # return false if there is no blacklist
+
+    for my $entry( @{ $self->blacklist || [] } ) {
+        for my $filter( values %{ $self->_list_filters($allowed) } ) {
+            next unless $entry =~ /$filter/; # if there is at least a letter at the beginning then this entry has a color we can check
+
+            my $capture = $1 || $entry;
+
+            return 1 if $val eq $capture;
+        }
+    }
+
+    return 0; # value is not in blacklist
+}
+
 sub _png_blocks_of_color {
     my $self = shift;
     my %args = ref $_[0] eq 'HASH' ? %{$_[0]} : @_;
@@ -210,34 +355,27 @@ sub _png_blocks_of_color {
     return @blocks;
 }
 
+sub _color_score {
+    my $self      = shift;
+    my ($c1, $c2) = @_;
+
+    return abs( $c1->[0] - $c2->[0] ) + abs( $c1->[1] - $c2->[1] ) + abs( $c1->[2] - $c2->[2] );
+}
+
 sub _find_lego_color {
-    my $self  = shift;
-    my $block = shift;
+    my $self        = shift;
+    my ($r, $g, $b) = @_;
 
-    my @optimal_color =
-            map  { $_->{'cid'} }
-            sort { $a->{'score'} <=> $b->{'score'} }
-            map  {
-                +{
-                    cid => $_->{'cid'},
-                    score => abs( $block->{'r'} - $_->{'rgb_color'}[0] )
-                           + abs( $block->{'g'} - $_->{'rgb_color'}[1] )
-                           + abs( $block->{'b'} - $_->{'rgb_color'}[2] ),
-                };
-            }
-            values %{ $self->lego_colors };
+    my %scores = map  {
+            $_->{'cid'} => $self->_color_score( [$r, $g, $b], $_->{'rgb_color'} ),
+        }
+        grep { ! $self->is_blacklisted( $_->{'cid'}, 'color' ) }
+        grep { $self->is_whitelisted( $_->{'cid'}, 'color' ) }
+        values %{ $self->lego_colors };
 
-    my ($optimal_color) = grep {
-         my $choose_this_color = 1;
+    my @optimal_color = sort { ($scores{$a} || 0) <=> ($scores{$b} || 0) } keys %scores;
 
-         $choose_this_color = 0 if ! $self->is_whitelisted( $_, 'color' );
-
-         $choose_this_color = 0 if $self->is_blacklisted( $_, 'color' );
-
-         $choose_this_color; # return result
-    } @optimal_color; # first color in list that passes whitelist and blacklist should be the optimal color for tested block
-
-    return $optimal_color;
+    return shift @optimal_color;
 }
 
 sub _approximate_lego_colors {
@@ -249,7 +387,7 @@ sub _approximate_lego_colors {
     my @colors;
 
     for my $block(@{ $args{'blocks'} }) {
-        push @colors, $self->_find_lego_color( $block );
+        push @colors, $self->_find_lego_color_fast( $block->{'r'}, $block->{'g'}, $block->{'b'} );
     }
 
     return @colors;
@@ -263,7 +401,7 @@ sub _generate_brick_list {
 
     my $unit_count   = scalar(@{ $args{'units'} });
     my @units        = @{ $args{'units'} };
-    my $row_width    = $self->block_row_width;
+    my $row_width    = $self->block_row_length;
     my $brick_height = 1; # bricks are only one unit high
     my @brick_list;
 
@@ -354,74 +492,22 @@ sub _list_filters {
     return $filters;
 }
 
-sub whitelist { shift->{'whitelist'} }
+sub _plan_info {
+    my $self = shift;
 
-sub has_whitelist {
-    my $self    = shift;
-    my $allowed = shift; # arrayref listing filters we can use
+    my %info;
 
-    my $found = 0;
-    for my $filter(values $self->_list_filters($allowed)) {
-        $found += scalar( grep { /$filter/ } @{ $self->whitelist || [] } );
-    }
-
-    return $found;
-}
-
-sub is_whitelisted {
-    my $self    = shift;
-    my $val     = shift;
-    my $allowed = shift; # arrayref listing filters we can use
-
-    return 1 if ! $self->has_whitelist($allowed); # return true if there is no whitelist
-
-    for my $entry( @{ $self->whitelist || [] } ) {
-        for my $filter( values %{ $self->_list_filters($allowed) } ) {
-            next unless $entry =~ /$filter/; # if there is at least a letter at the beginning then this entry has a color we can check
-
-            my $capture = $entry;
-            $capture =~ s/$filter/$1/;
-
-            return 1 if $val eq $capture;
+    for my $type (qw/metric imperial/) {
+        if ($self->{$type}) {
+            $info{$type} = {
+                depth  => $self->{'brick_depth'} * $self->lego_dimensions->{$type}->{'lego_unit_depth'},
+                length => $self->block_row_length * $self->lego_dimensions->{$type}->{'lego_unit_length'},
+                height => ($self->block_row_height * $self->lego_dimensions->{$type}->{'lego_unit_height'}) + $self->lego_dimensions->{$type}->{'lego_unit_stud_height'},
+            };
         }
     }
 
-    return 0; # value is not in whitelist
-}
-
-sub blacklist { shift->{'blacklist'} }
-
-sub has_blacklist {
-    my $self    = shift;
-    my $allowed = shift; # optional filter restriction
-
-    my $found = 0;
-
-    for my $filter(values $self->_list_filters($allowed)) {
-        $found += scalar( grep { /$filter/ } @{ $self->blacklist || [] } );
-    }
-
-    return $found;
-}
-
-sub is_blacklisted {
-    my $self    = shift;
-    my $val     = shift;
-    my $allowed = shift; # optional filter restriction
-
-    return 0 if ! $self->has_blacklist($allowed); # return false if there is no blacklist
-
-    for my $entry( @{ $self->blacklist || [] } ) {
-        for my $filter( values %{ $self->_list_filters($allowed) } ) {
-            next unless $entry =~ /$filter/; # if there is at least a letter at the beginning then this entry has a color we can check
-
-            my $capture = $1 || $entry;
-
-            return 1 if $val eq $capture;
-        }
-    }
-
-    return 0; # value is not in blacklist
+    return \%info;
 }
 
 =pod
@@ -477,7 +563,7 @@ $hash->{'filename'} = $args{'filename'};
     brick_depth - Optional. The depth of all generated bricks. Defaults to 1.
         e.g. brick_depth => 2 # final depth of all bricks are 2. So 2 x length x height
 
-    brick_heigtht - Optional. The height of all generated bricks. Defaults to 1.
+    brick_height - Optional. The height of all generated bricks. Defaults to 1.
         e.g. brick_height => 2 # final height of all bricks are 2. So depth x length x 2
 
     whitelist - Optional. Array ref of colors, dimensions or color and dimensions that are allowed in the final plan output.
@@ -491,12 +577,24 @@ $hash->{'filename'} = $args{'filename'};
  Comment   :
  See Also  :
 
+=head2 lego_dimensions
+
+ Usage     : ->lego_dimensions()
+ Purpose   : returns a hashref with lego dimension information in millimeters (metric) or inches (imperial)
+
+ Returns   : hashref with lego dimension information, millimeters is default
+ Argument  : $type - if set to imperial then dimension information is returned in inches
+ Throws    :
+
+ Comment   :
+ See Also  :
+
 =head2 lego_colors
 
  Usage     : ->lego_colors()
- Purpose   : Returns lego color constants consolidated as a hash.
+ Purpose   : returns lego color constants consolidated as a hash.
 
- Returns   : Hash ref with color constants keyed by the official color name in key form.
+ Returns   : hashref with color constants keyed by the official color name in key form.
  Argument  :
  Throws    :
 
@@ -540,9 +638,9 @@ $hash->{'filename'} = $args{'filename'};
  Comment   :
  See Also  :
 
-=head2 block_row_width
+=head2 block_row_length
 
- Usage     : ->block_row_width()
+ Usage     : ->block_row_length()
  Purpose   : Return the width of one row of blocks. Since a block list is a single dimension array this is useful to figure out whict row a block is on.
 
  Returns   : The length of a row of blocks (image width / unit size)
@@ -552,6 +650,18 @@ $hash->{'filename'} = $args{'filename'};
  Comment   :
  See Also  :
 
+=head2 block_row_height
+
+ Usage     : ->block_row_height()
+ Purpose   : Return the height in blocks.
+
+ Returns   : The height of a row of blocks (image height / unit size)
+ Argument  :
+ Throws    :
+
+ Comment   :
+ See Also  :
+
 =head2 process
 
  Usage     : ->process()
@@ -565,66 +675,6 @@ $hash->{'filename'} = $args{'filename'};
  Comment   :
  See Also  :
 
-=head2 _png_blocks_of_color
-
- Usage     : ->_png_blocks_of_color()
- Purpose   : Convert a provided PNG into a list of rgb values based on [row][color]. Size of blocks are determined by 'unit_size'
-
- Returns   : A list of hashes contain r, g and b values. e.g. ( { r => #, g => #, b => # }, { ... }, ... )
- Argument  :
- Throws    :
-
- Comment   :
- See Also  :
-
-=head2 _find_lego_color
-
- Usage     : ->_find_lego_color
- Purpose   : given an rgb hash, finds the optimal lego color
-
- Returns   : A lego color common name key that can then reference lego color information using L<Lego::From::PNG::lego_colors>
- Argument  :
- Throws    :
-
- Comment   :
- See Also  :
-
-=head2 _approximate_lego_colors
-
- Usage     : ->_approximate_lego_colors()
- Purpose   : Generate a list of lego colors based on a list of blocks ( array of hashes containing rgb values )
-
- Returns   : A list of lego color common name keys that can then reference lego color information using L<Lego::From::PNG::lego_colors>
- Argument  :
- Throws    :
-
- Comment   :
- See Also  :
-
-=head2 _generate_brick_list
-
- Usage     : ->_approximate_lego_colors()
- Purpose   : Generate a list of lego colors based on a list of blocks ( array of hashes containing rgb values )
-
- Returns   : A list of lego color common name keys that can then reference lego color information using L<Lego::From::PNG::lego_colors>
- Argument  :
- Throws    :
-
- Comment   :
- See Also  :
-
-=head2 _list_filters
-
- Usage     : ->_list_filters()
- Purpose   : return whitelist/blacklist filters
-
- Returns   : an hashref of filters
- Argument  : an optional filter restriction to limit set of filters returned to just one
- Throws    :
-
- Comment   :
- See Also  :
-
 =head2 whitelist
 
  Usage     : ->whitelist()
@@ -697,6 +747,81 @@ $hash->{'filename'} = $args{'filename'};
  Comment   :
  See Also  :
 
+=head2 _png_blocks_of_color
+
+ Usage     : ->_png_blocks_of_color()
+ Purpose   : Convert a provided PNG into a list of rgb values based on [row][color]. Size of blocks are determined by 'unit_size'
+
+ Returns   : A list of hashes contain r, g and b values. e.g. ( { r => #, g => #, b => # }, { ... }, ... )
+ Argument  :
+ Throws    :
+
+ Comment   :
+ See Also  :
+
+ =head2  _color_score
+
+ Usage     : ->_color_score()
+ Purpose   : returns a score indicating the likeness of one color to another. The lower the number the closer the colors are to each other.
+
+ Returns   : returns a positive integer score
+ Argument  : $c1 - array ref with rgb color values in that order
+             $c2 - array ref with rgb color values in that order
+ Throws    :
+
+ Comment   :
+ See Also  :
+
+=head2 _find_lego_color
+
+ Usage     : ->_find_lego_color
+ Purpose   : given an rgb params, finds the optimal lego color
+
+ Returns   : A lego color common name key that can then reference lego color information using L<Lego::From::PNG::lego_colors>
+ Argument  : $r - the red value of a color
+             $g - the green value of a color
+             $b - the blue value of a color
+ Throws    :
+
+ Comment   : this subroutine is memoized as the name _find_lego_color_fast
+ See Also  :
+
+=head2 _approximate_lego_colors
+
+ Usage     : ->_approximate_lego_colors()
+ Purpose   : Generate a list of lego colors based on a list of blocks ( array of hashes containing rgb values )
+
+ Returns   : A list of lego color common name keys that can then reference lego color information using L<Lego::From::PNG::lego_colors>
+ Argument  :
+ Throws    :
+
+ Comment   :
+ See Also  :
+
+=head2 _generate_brick_list
+
+ Usage     : ->_approximate_lego_colors()
+ Purpose   : Generate a list of lego colors based on a list of blocks ( array of hashes containing rgb values )
+
+ Returns   : A list of lego color common name keys that can then reference lego color information using L<Lego::From::PNG::lego_colors>
+ Argument  :
+ Throws    :
+
+ Comment   :
+ See Also  :
+
+=head2 _list_filters
+
+ Usage     : ->_list_filters()
+ Purpose   : return whitelist/blacklist filters
+
+ Returns   : an hashref of filters
+ Argument  : an optional filter restriction to limit set of filters returned to just one
+ Throws    :
+
+ Comment   :
+ See Also  :
+
 =head1 BUGS
 
 =head1 SUPPORT
@@ -724,4 +849,3 @@ perl(1).
 =cut
 
 1;
-
