@@ -1,7 +1,7 @@
 package Lego::From::PNG::Const;
 
 BEGIN {
-    $Lego::From::PNG::Const::VERSION = '0.01';
+    $Lego::From::PNG::Const::VERSION = '0.03';
 }
 
 require Exporter;
@@ -9,6 +9,17 @@ require Exporter;
 @EXPORT_OK = qw(
     LEGO_COLORS
     LEGO_BRICK_LENGTHS
+    LEGO_UNIT
+    LEGO_UNIT_LENGTH
+    LEGO_UNIT_DEPTH
+    LEGO_UNIT_HEIGHT
+    LEGO_UNIT_STUD_DIAMETER
+    LEGO_UNIT_STUD_HEIGHT
+    LEGO_UNIT_STUD_SPACING
+    LEGO_UNIT_EDGE_TO_STUD
+    MILLIMETER_TO_INCH
+    METRIC_SUFFIX
+    IMPERIAL_SUFFIX
     AQUA_OFFICIAL_NAME
     AQUA_COMMON_NAME
     AQUA_HEX_COLOR
@@ -265,6 +276,7 @@ require Exporter;
 
 %EXPORT_TAGS = ('all' => \@EXPORT_OK);
 
+# List of basic lego colors
 use constant LEGO_COLORS => qw(
     AQUA
     BLACK
@@ -310,11 +322,13 @@ use constant LEGO_COLORS => qw(
     WHITE_GLOW
 );
 
+# Common Brick Dimensions (by stud count)
 use constant LEGO_BRICK_LENGTHS => qw(
     1
     2
     3
     4
+    5
     6
     8
     10
@@ -322,6 +336,23 @@ use constant LEGO_BRICK_LENGTHS => qw(
     16
 );
 
+# Lego Dimensions Info
+use constant {
+    LEGO_UNIT               => 1.6, # in millimeters
+    # the rest of these values are in lego units
+    LEGO_UNIT_LENGTH        => 5,
+    LEGO_UNIT_DEPTH         => 5,
+    LEGO_UNIT_HEIGHT        => 6,
+    LEGO_UNIT_STUD_DIAMETER => 3,
+    LEGO_UNIT_STUD_HEIGHT   => 1,
+    LEGO_UNIT_STUD_SPACING  => 2, # space between the edge of two lego studs on a brick
+    LEGO_UNIT_EDGE_TO_STUD  => 1, # distance from the edge of a brick to the edge of a stud (this is why stud spacing is double that)
+    MILLIMETER_TO_INCH      => 0.0393701, # just so we can provide a conversion option
+    METRIC_SUFFIX           => "mm",
+    IMPERIAL_SUFFIX         => "in.",
+};
+
+# Color Info
 use constant {
     AQUA_OFFICIAL_NAME => 'Aqua',
     AQUA_COMMON_NAME => 'Unikitty Blue',
